'use strict';
const Schema = require('mongoose').Schema;
const moment = require('moment');
const metaPlugin = require('naf-framework-mongoose/lib/model/meta-plugin');
// 发货数据
const sendList = new Schema({
  split_id: { type: String, maxLength: 200 }, // 拆分的货物id
  no: { type: String, maxLength: 200 }, // 运输单号
  name: { type: String, maxLength: 200 }, // 货物名
  number: { type: Number, maxLength: 200 }, // 数量
  weight: { type: Number, maxLength: 200 }, // 重量
  volume: { type: Number, maxLength: 200 }, // 体积
  time: { type: String, maxLength: 200, default: moment().format('YYYY-MM-DD HH:mm:ss') }, // 收货时间
  remark: { type: String, maxLength: 200 },
});
// 签收数据
const arriveList = new Schema({
  split_id: { type: String, maxLength: 200 }, // 拆分的货物id
  no: { type: String, maxLength: 200 }, // 运输单号
  name: { type: String, maxLength: 200 }, // 货物名
  number: { type: Number, maxLength: 200 }, // 数量
  weight: { type: Number, maxLength: 200 }, // 重量
  volume: { type: Number, maxLength: 200 }, // 体积
  time: { type: String, maxLength: 200, default: moment().format('YYYY-MM-DD HH:mm:ss') }, // 收货时间
  remark: { type: String, maxLength: 200 },
});
// 货物列表
const goods = new Schema({
  name: { type: String, maxLength: 200 }, // 货物名
  number: { type: Number, maxLength: 200 }, // 数量
  weight: { type: Number, maxLength: 200 }, // 重量
  volume: { type: Number, maxLength: 200 }, // 体积
  transport_type: { type: String, maxLength: 200, default: '直达' }, // 该货物的运输类型: 直达;转运
  remark: { type: String, maxLength: 200 },
});

// 拆分货物
// 刚生成订单,货物数据同时进入货物列表和拆分列表时,type为0;
// 之后所有拆分生成的数据,type都是1
const splitGoods = new Schema({
  name: { type: String, maxLength: 200 }, // 货物名
  number: { type: Number, maxLength: 200, default: 0 }, // 数量
  weight: { type: Number, maxLength: 200 }, // 重量
  volume: { type: Number, maxLength: 200 }, // 体积
  transport_type: { type: String, maxLength: 200, default: '0' }, // 该货物的运输类型: 0=>直达;1=>转运
  remark: { type: String, maxLength: 200 }, // remark属于拆分过后的,只有原货物进行复制,拆分后的不需要复制,可以自己编辑
  pid: { type: String, maxLength: 200, required: true }, // 来源id;type为0时,从goods中去找;1时,就都是这个文档的事了
  type: { type: String, maxLength: 200, default: '0' }, // 0=>货物列表原始数据;1=>拆分数据;
  status: { type: String, maxLength: 200, default: '0' }, // -1=>已到达; 0=>未装车; 1=>运输中; 2=>转运地签收; 3=>转运中
});

// 订单记录
const recordList = new Schema({
  opera: { type: String, maxLength: 200 }, // 操作人
  operaid: { type: String, maxLength: 200 }, // 操作人id
  message: { type: String, maxLength: 200 }, // 操作
  time: { type: String, default: moment().format('YYYY-MM-DD HH:mm:ss') }, // 操作时间
});

// 货物收入
const goods_bill = new Schema({
  name: { type: String, maxlength: 200 }, // 货物名称
  mode: { type: String, maxlength: 200 }, // 计费方式
  number: { type: Number, maxLength: 200 }, // 数量
  weight: { type: Number, maxLength: 200 }, // 重量
  volume: { type: Number, maxLength: 200 }, // 体积
  price: { type: Number, maxLength: 200 }, // 单价
  taxes: { type: String, maxLength: 200 }, // 税率
  sq_ys: { type: Number, maxLength: 200 }, // 税前应收
  sq_ss: { type: Number, maxLength: 200 }, // 税前实收
  sh_ys: { type: Number, maxLength: 200 }, // 税后应收
  sh_ss: { type: Number, maxLength: 200 }, // 税后实收
});

// 收入/支出项
const cost_bill = new Schema({
  item: { type: String, maxlength: 200 }, // 支出项
  taxes: { type: String, maxLength: 200 }, // 税率
  sq_ys: { type: Number, maxLength: 200 }, // 税前应收
  sq_ss: { type: Number, maxLength: 200 }, // 税前实收
  sh_ys: { type: Number, maxLength: 200 }, // 税后应收
  sh_ss: { type: Number, maxLength: 200 }, // 税后实收
});

// 订单
const order = {
  order_no: {
    type: String,
    maxLength: 200,
    field: { label: '订单号', filter: true },
  },
  owner: {
    type: String,
    required: true,
    maxLength: 200,
    field: { label: '订单创建人' },
  },
  principal: {
    type: String,
    maxLength: 200,
    field: { label: '当前订单负责人' },
  },
  client: {
    type: String,
    required: true,
    maxLength: 200,
    field: { label: '客户' },
  },
  treaty: {
    type: String,
    maxLength: 200,
    field: { label: '合同' },
  },
  item: {
    type: String,
    maxLength: 200,
    field: { label: '项目' },
  },
  route: {
    type: String,
    maxLength: 200,
    field: { label: '线路' },
  },
  rq_send_time: {
    type: String,
    maxLength: 200,
    field: { label: '要求发货日期' },
  },
  send_time: {
    type: [ sendList ],
    maxLength: 200,
    field: { label: '发货日期' },
  },
  rq_arrive_time: {
    type: String,
    maxLength: 200,
    field: { label: '要求到达日期' },
  },
  // 到达日期有问题:应该有多种货物不同到达日期 或 某种货物被拆分,不同的到达日期
  // 但是这并不影响要求 到达日期
  arrive_time: {
    type: [ arriveList ],
    maxLength: 200,
    field: { label: '到达日期' },
  },
  pre_money: {
    type: Number,
    maxLength: 200,
    default: 0,
    field: { label: '预收金额', type: 'number' },
  },
  goods: {
    type: [ goods ],
    maxLength: 200,
    field: {
      label: '货物',
      remark: '只用来记录货物最开始的情况,一旦进行拆分,则不允许更改',
    },
  },
  split: {
    type: [ splitGoods ],
    maxLength: 200,
    field: {
      label: '货物拆分列表',
      remark: '货物录入后,同时进入这里,这里的货物供拆分使用',
    },
  },
  goods_bill: {
    type: [ goods_bill ],
    maxLength: 200,
    field: { label: '货物收入单据' },
  },
  in_bill: {
    type: [ cost_bill ],
    maxLength: 200,
    field: { label: '收入-服务单据' },
  },
  out_bill: {
    type: [ cost_bill ],
    maxLength: 200,
    field: { label: '支出-服务单据' },
  },
  // 运输单与订单的关系: 订单的拆分货物id会在运输单的货物列表中
  is_js: {
    type: Boolean,
    maxLength: 200,
    default: false,
    field: {
      label: '结算状态',
      filter: 'select',
      type: 'radio',
      format: (i => (i ? '已结算' : '未结算')).toString(),
      list: [
        { label: '未结算', value: false },
        { label: '已结算', value: true },
      ],
    },
  },
  goods_status: {
    type: String,
    maxLength: 200,
    default: '未发车',
    field: {
      label: '货物状态',
      filter: 'select',
      type: 'text',
      remark: `查看拆分列表中
        1)所有的货物状态都为0 => 未发车
        2)至少1个0,其余全为1||全为2||全为3,没有-1=> 部分货物未发车 ; 部分运输中
        3)至少1个0,至少1个1||至少1个2||至少1个3,至少1个-1=> 部分货物未发车 ; 部分运输中 ; 部分货物已到达
        4)至少1个0,没有1&&没有2&&没有3,其余全为-1=> 部分货物未发车 ; 部分货物已到达
        5)没有0,全为1||全为2||全为3,没有-1 => 所有货物运输中
        4)没有0,至少1个1||至少1个2||至少1个3,其余全为-1 => 所有货物运输中 ; 部分货物已到达
        6)没有0,没有1&&没有2&&没有3,全为-1 => 所有货物已到达
      `,
    },
  },
  status: {
    type: String,
    maxLength: 200,
    default: '0',
    field: {
      label: '数据状态',
      filter: 'select',
      type: 'radio',
      format: (i => (i === '0' ? '正常' : '冻结')).toString(),
      list: [
        { label: '正常', value: '0' },
        { label: '冻结', value: '1' },
      ],
      remark: '冻结时不允许操作和此单有关的信息,运输改变货物及订单状态部分除外',
    },
  }, // 状态:0=>使用;1禁用
  remark: {
    type: String,
    maxLength: 200,
    field: {
      label: '备注',
      type: 'textarea',
      options: { autosize: { minRows: 3, maxRows: 5 } },
    },
  },
  record: {
    type: [ recordList ],
    field: { label: '订单记录' },
  },
};

const schema = new Schema(order, { toJSON: { virtuals: true } });
schema.index({ id: 1 });
schema.plugin(metaPlugin);

module.exports = app => {
  const { mongoose } = app;
  return mongoose.model('Order', schema, 'order');
};
